"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChunkBuilder = exports.groupChunks = void 0;
const tslib_1 = require("tslib");
const generator_1 = require("../const/generator");
const file_1 = require("../const/file");
function whichFamily(type) {
    const idx = file_1.FILE_TYPE_FAMILY.findIndex((family) => family.indexOf(type) >= 0);
    if (idx < 0) {
        return undefined;
    }
    return [idx, file_1.FILE_TYPE_FAMILY[idx]];
}
const groupChunks = (chunks) => {
    const tmp = {};
    const col = chunks.reduce((chunksSet, chunk) => {
        const fileKey = chunk.subModule || generator_1.COMMON_SUB_MODULE_NAME;
        if (!chunksSet[fileKey]) {
            // eslint-disable-next-line no-param-reassign
            chunksSet[fileKey] = [];
        }
        const res = whichFamily(chunk.fileType);
        const info = {
            chunk,
        };
        if (res) {
            const [familyIdx, family] = res;
            const rank = family.indexOf(chunk.fileType);
            if (tmp[fileKey]) {
                if (tmp[fileKey][familyIdx] !== undefined) {
                    if (tmp[fileKey][familyIdx] > rank) {
                        tmp[fileKey][familyIdx] = rank;
                    }
                }
                else {
                    tmp[fileKey][familyIdx] = rank;
                }
            }
            else {
                tmp[fileKey] = {};
                tmp[fileKey][familyIdx] = rank;
            }
            info.familyIdx = familyIdx;
        }
        chunksSet[fileKey].push(info);
        return chunksSet;
    }, {});
    const result = [];
    Object.keys(col).forEach((key) => {
        const byType = {};
        col[key].forEach((info) => {
            let t = info.chunk.fileType;
            if (info.familyIdx !== undefined) {
                t = file_1.FILE_TYPE_FAMILY[info.familyIdx][tmp[key][info.familyIdx]];
                // eslint-disable-next-line no-param-reassign
                info.chunk.fileType = t;
            }
            if (!byType[t]) {
                byType[t] = [];
            }
            byType[t].push(info.chunk);
        });
        result.push(...Object.keys(byType).map((t) => byType[t]));
    });
    return result;
};
exports.groupChunks = groupChunks;
/**
 * 代码片段构建器
 *
 * @export
 * @class ChunkBuilder
 * @template T
 */
class ChunkBuilder {
    constructor(plugins = []) {
        this.plugins = plugins;
    }
    run(ir, initialStructure = {
        ir,
        chunks: [],
        depNames: [],
        contextData: {},
    }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const structure = initialStructure;
            const finalStructure = yield this.plugins.reduce((previousPluginOperation, plugin) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const modifiedStructure = yield previousPluginOperation;
                return plugin(modifiedStructure);
            }), Promise.resolve(structure));
            const chunks = (0, exports.groupChunks)(finalStructure.chunks);
            return {
                chunks,
            };
        });
    }
    getPlugins() {
        return this.plugins;
    }
    addPlugin(plugin) {
        this.plugins.push(plugin);
    }
}
exports.ChunkBuilder = ChunkBuilder;
exports.default = ChunkBuilder;
//# sourceMappingURL=ChunkBuilder.js.map