"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSubDirectoriesSync = exports.colorizeDiffOutput = exports.ensureShellExec = exports.diffActualAndExpectedSync = exports.runPrettierSync = exports.removeActualDirRecursiveSync = exports.createDiskPublisher = void 0;
const tslib_1 = require("tslib");
const child_process_1 = require("child_process");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
var disk_1 = require("../publisher/disk");
Object.defineProperty(exports, "createDiskPublisher", { enumerable: true, get: function () { return disk_1.createDiskPublisher; } });
function removeActualDirRecursiveSync(actualDir, caseFullDir) {
    ensureShellExec('rm', ['-rf', actualDir], { cwd: caseFullDir });
}
exports.removeActualDirRecursiveSync = removeActualDirRecursiveSync;
function runPrettierSync(files, cwd) {
    ensureShellExec('npx', ['prettier', '--write', ...files], { cwd });
}
exports.runPrettierSync = runPrettierSync;
function diffActualAndExpectedSync(caseFullDir) {
    const res = (0, child_process_1.spawnSync)('diff', ['-wBur', '-x', '.eslintrc.js', '-x', 'abc.json', 'expected', 'actual'], {
        cwd: caseFullDir,
        stdio: 'pipe',
        shell: true,
        encoding: 'utf-8',
    });
    return colorizeDiffOutput(res.stdout);
}
exports.diffActualAndExpectedSync = diffActualAndExpectedSync;
function ensureShellExec(shellCmd, args, { cwd = process.cwd() } = {}) {
    const res = (0, child_process_1.spawnSync)(shellCmd, args, {
        stdio: 'pipe',
        shell: true,
        cwd,
        encoding: 'utf-8',
    });
    if (res.status !== 0) {
        throw new Error(`Shell command "${shellCmd} ${args.slice(0, 2).join(' ')}..." failed with status: ${res.status} (Full command: "${shellCmd} ${args.join(' ')}" ) (stderr: ${res.stderr})`);
    }
    return res;
}
exports.ensureShellExec = ensureShellExec;
function colorizeDiffOutput(output) {
    if (!output) {
        return output;
    }
    return output
        .split('\n')
        .map((line) => {
        if (/^Only/i.test(line)) {
            return chalk_1.default.red(line);
        }
        else if (line[0] === '+') {
            return chalk_1.default.yellow(line);
        }
        else if (line[0] === '-') {
            return chalk_1.default.red(line);
        }
        else {
            return line;
        }
    })
        .join('\n');
}
exports.colorizeDiffOutput = colorizeDiffOutput;
function getSubDirectoriesSync(baseDir) {
    return fs_1.default
        .readdirSync(baseDir)
        .filter((dirOrFileName) => fs_1.default.statSync(path_1.default.join(baseDir, dirOrFileName)).isDirectory());
}
exports.getSubDirectoriesSync = getSubDirectoriesSync;
expect.extend({
    toBeSameFileContents(caseFullDir) {
        const differences = diffActualAndExpectedSync(caseFullDir);
        if (differences) {
            return {
                message: () => differences,
                pass: false,
            };
        }
        else {
            return {
                message: () => `expected case [${caseFullDir}] not to be same`,
                pass: true,
            };
        }
    },
});
//# sourceMappingURL=solutionHelper.js.map