"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const change_case_1 = tslib_1.__importDefault(require("change-case"));
const types_1 = require("../../types");
const generator_1 = require("../../const/generator");
const pluginFactory = () => {
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const ir = next.ir;
        const { chunks } = next;
        if (ir && ir.deps && ir.deps.length > 0) {
            let lowcodeMaterialsStyleAdded = false;
            let fusionUIStyleAdded = false;
            let nextStyleAddedMap = {};
            ir.deps.forEach((dep) => {
                if (dep.package === '@alifd/next' && !nextStyleAddedMap[dep.exportName]) {
                    chunks.push({
                        type: types_1.ChunkType.STRING,
                        fileType: types_1.FileType.JSX,
                        name: generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
                        content: `import '@alifd/next/lib/${change_case_1.default.paramCase(dep.exportName)}/style';`,
                        linkAfter: [generator_1.COMMON_CHUNK_NAME.ExternalDepsImport],
                    });
                    nextStyleAddedMap[dep.exportName] = true;
                }
                else if (dep.package === '@alilc/lowcode-materials' && !lowcodeMaterialsStyleAdded) {
                    chunks.push({
                        type: types_1.ChunkType.STRING,
                        fileType: types_1.FileType.JSX,
                        name: generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
                        content: 'import \'@alilc/lowcode-materials/lib/style\';',
                        linkAfter: [generator_1.COMMON_CHUNK_NAME.ExternalDepsImport],
                    });
                    lowcodeMaterialsStyleAdded = true;
                }
                else if (dep.package === '@alifd/fusion-ui' && !fusionUIStyleAdded) {
                    chunks.push({
                        type: types_1.ChunkType.STRING,
                        fileType: types_1.FileType.JSX,
                        name: generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
                        content: 'import \'@alifd/fusion-ui/lib/style\';',
                        linkAfter: [generator_1.COMMON_CHUNK_NAME.ExternalDepsImport],
                    });
                    fusionUIStyleAdded = true;
                }
            });
        }
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=styleImport.js.map