"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const generator_1 = require("../../../const/generator");
const types_1 = require("../../../types");
const pathHelper_1 = require("../../../utils/pathHelper");
const pluginFactory = (config) => {
    const cfg = Object.assign({ fileType: types_1.FileType.JSX }, config);
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const next = Object.assign({}, pre);
        const ir = next.ir;
        next.contextData.useRefApi = true;
        const useRef = !!((_a = ir.analyzeResult) === null || _a === void 0 ? void 0 : _a.isUsingRef);
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
            content: `
        import utils${useRef ? ', { RefsManager }' : ''} from '${(0, pathHelper_1.getSlotRelativePath)({ contextData: next.contextData, from: 'components', to: 'utils' })}';
      `,
            linkAfter: [generator_1.COMMON_CHUNK_NAME.ExternalDepsImport],
        });
        if (cfg.preferClassProperty) {
            // mode: class property
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsVar,
                content: 'utils = utils;',
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.InsVar]],
            });
        }
        else {
            // mode: assign in constructor
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.ConstructorContent,
                content: 'this.utils = utils;',
                linkAfter: [generator_1.FC_DEFINE_CHUNK_NAME.ConstructorStart],
            });
        }
        if (useRef) {
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.ConstructorContent,
                content: 'this._refsManager = new RefsManager();',
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.ConstructorContent]],
            });
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsMethod,
                content: `
          $ = (refName) => {
            return this._refsManager.get(refName);
          }
        `,
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.InsMethod]],
            });
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsMethod,
                content: `
          $$ = (refName) => {
            return this._refsManager.getAll(refName);
          }
        `,
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.InsMethod]],
            });
        }
        else {
            // useRef 为 false 的时候是指没有组件在 props 中配置 ref 属性，但这个时候其实也可能有代码访问 this.$/$$ 所以还是加上个空的代码
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsMethod,
                content: ' $ = () => null; ',
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.InsMethod]],
            });
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsMethod,
                content: ' $$ = () => [];        ',
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.InsMethod]],
            });
        }
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=containerInjectUtils.js.map