"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const generator_1 = require("../../../const/generator");
const compositeType_1 = require("../../../utils/compositeType");
const Scope_1 = require("../../../utils/Scope");
const types_1 = require("../../../types");
const pluginFactory = (config) => {
    const cfg = Object.assign({ fileType: types_1.FileType.JSX, implementType: 'insMember' }, config);
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const ir = next.ir;
        const scope = Scope_1.Scope.createRootScope();
        const state = ir.state || {};
        const fields = Object.keys(state).map((stateName) => {
            // TODO: 这里用什么 handlers?
            const value = (0, compositeType_1.generateCompositeType)(state[stateName], scope);
            return `${JSON.stringify(stateName)}: ${value}`;
        });
        if (cfg.implementType === 'inConstructor') {
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorContent,
                content: `this.state = { ${fields.join(',')} };`,
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorContent]],
            });
        }
        else if (cfg.implementType === 'insMember') {
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsVar,
                content: `state = { ${fields.join(',')} };`,
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.InsVar]],
            });
        }
        // TODO: hooks state??
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=containerInitState.js.map