"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const change_case_1 = tslib_1.__importDefault(require("change-case"));
const generator_1 = require("../../../const/generator");
const const_1 = require("./const");
const types_1 = require("../../../types");
const validate_1 = require("../../../utils/validate");
const pluginFactory = () => {
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const ir = next.ir;
        // 将模块名转换成 PascalCase 的格式，并添加特定后缀，防止命名冲突
        const componentClassName = (0, validate_1.ensureValidClassName)(`${change_case_1.default.pascalCase(ir.moduleName)}$$${ir.containerType}`);
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.Start,
            content: `class ${componentClassName} extends React.Component {`,
            linkAfter: [
                generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
                generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
                generator_1.COMMON_CHUNK_NAME.ImportAliasDefine,
                generator_1.COMMON_CHUNK_NAME.FileVarDefine,
                generator_1.COMMON_CHUNK_NAME.FileUtilDefine,
            ],
        });
        if (ir.containerType === 'Component') {
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: types_1.FileType.JSX,
                name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsVar,
                content: `static displayName = '${ir.moduleName}';`,
                linkAfter: [
                    generator_1.CLASS_DEFINE_CHUNK_NAME.Start,
                ],
            });
        }
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.End,
            content: '}',
            linkAfter: [
                ...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.End],
                const_1.REACT_CHUNK_NAME.ClassRenderEnd,
                const_1.REACT_CHUNK_NAME.ClassDidMountEnd,
            ],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorStart,
            content: 'constructor(props, context) { super(props); ',
            linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorStart]],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorEnd,
            content: '}',
            linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorEnd]],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: const_1.REACT_CHUNK_NAME.ClassDidMountStart,
            content: 'componentDidMount() {',
            linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.End]],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: const_1.REACT_CHUNK_NAME.ClassDidMountEnd,
            content: '}',
            linkAfter: [const_1.REACT_CHUNK_NAME.ClassDidMountContent, const_1.REACT_CHUNK_NAME.ClassDidMountStart],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: const_1.REACT_CHUNK_NAME.ClassRenderStart,
            content: 'render() {',
            linkAfter: [
                ...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.End],
                const_1.REACT_CHUNK_NAME.ClassDidMountEnd,
            ],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: const_1.REACT_CHUNK_NAME.ClassRenderEnd,
            content: '}',
            linkAfter: [
                const_1.REACT_CHUNK_NAME.ClassRenderStart,
                const_1.REACT_CHUNK_NAME.ClassRenderPre,
                const_1.REACT_CHUNK_NAME.ClassRenderJSX,
            ],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: generator_1.COMMON_CHUNK_NAME.FileExport,
            content: `export default ${componentClassName};`,
            linkAfter: [
                generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
                generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
                generator_1.COMMON_CHUNK_NAME.ImportAliasDefine,
                generator_1.COMMON_CHUNK_NAME.FileVarDefine,
                generator_1.COMMON_CHUNK_NAME.FileUtilDefine,
                generator_1.CLASS_DEFINE_CHUNK_NAME.End,
            ],
        });
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=containerClass.js.map