"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const generator_1 = require("../../../const/generator");
const const_1 = require("./const");
const jsExpression_1 = require("../../../utils/jsExpression");
const types_1 = require("../../../types");
const lowcode_types_1 = require("@alilc/lowcode-types");
const common_1 = require("../../../utils/common");
const pluginFactory = (config) => {
    const cfg = Object.assign({ fileType: types_1.FileType.JSX, exportNameMapping: {}, normalizeNameMapping: {} }, config);
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const ir = next.ir;
        if (ir.lifeCycles) {
            const { lifeCycles } = ir;
            const chunks = Object.keys(lifeCycles).map((lifeCycleName) => {
                var _a;
                // 过滤掉非法数据（有些场景下会误传入空字符串或 null)
                if (!(0, lowcode_types_1.isJSFunction)(lifeCycles[lifeCycleName]) &&
                    !(0, common_1.isJSExpressionFn)(lifeCycles[lifeCycleName]) &&
                    !(0, lowcode_types_1.isJSExpression)(lifeCycles[lifeCycleName])) {
                    return null;
                }
                let normalizeName;
                // constructor会取到对象的构造函数
                if (lifeCycleName === 'constructor') {
                    normalizeName = lifeCycleName;
                }
                else {
                    normalizeName = cfg.normalizeNameMapping[lifeCycleName] || lifeCycleName;
                }
                if ((_a = cfg === null || cfg === void 0 ? void 0 : cfg.exclude) === null || _a === void 0 ? void 0 : _a.includes(normalizeName)) {
                    return null;
                }
                const exportName = cfg.exportNameMapping[lifeCycleName] || lifeCycleName;
                if (normalizeName === 'constructor') {
                    return {
                        type: types_1.ChunkType.STRING,
                        fileType: cfg.fileType,
                        name: generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorContent,
                        content: (0, jsExpression_1.generateFunction)(lifeCycles[lifeCycleName], { isBlock: true }),
                        linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorStart]],
                    };
                }
                if (normalizeName === 'componentDidMount') {
                    return {
                        type: types_1.ChunkType.STRING,
                        fileType: cfg.fileType,
                        name: const_1.REACT_CHUNK_NAME.ClassDidMountContent,
                        content: (0, jsExpression_1.generateFunction)(lifeCycles[lifeCycleName], { isBlock: true }),
                        linkAfter: [const_1.REACT_CHUNK_NAME.ClassDidMountStart],
                    };
                }
                if (normalizeName === 'render') {
                    return {
                        type: types_1.ChunkType.STRING,
                        fileType: cfg.fileType,
                        name: const_1.REACT_CHUNK_NAME.ClassRenderPre,
                        content: (0, jsExpression_1.generateFunction)(lifeCycles[lifeCycleName], { isBlock: true }),
                        linkAfter: [const_1.REACT_CHUNK_NAME.ClassRenderStart],
                    };
                }
                return {
                    type: types_1.ChunkType.STRING,
                    fileType: cfg.fileType,
                    name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsMethod,
                    content: (0, jsExpression_1.generateFunction)(lifeCycles[lifeCycleName], {
                        name: exportName,
                        isMember: true,
                    }),
                    linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.InsMethod]],
                };
            }).filter((i) => !!i);
            next.chunks.push(...chunks.filter((x) => x !== null));
        }
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=containerLifeCycle.js.map