"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const types_1 = require("../../../types");
const const_1 = require("./const");
const generator_1 = require("../../../const/generator");
const nodeToJSX_1 = require("../../../utils/nodeToJSX");
const Scope_1 = require("../../../utils/Scope");
const jsExpression_1 = require("../../../utils/jsExpression");
const transformJsExpression_1 = require("../../../core/jsx/handlers/transformJsExpression");
const transformThis2Context_1 = require("../../../core/jsx/handlers/transformThis2Context");
const compositeType_1 = require("../../../utils/compositeType");
const pluginFactory = (config) => {
    const cfg = Object.assign({ fileType: types_1.FileType.JSX, 
        // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
        nodeTypeMapping: {} }, config);
    const { nodeTypeMapping } = cfg;
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const { tolerateEvalErrors = true, evalErrorsHandler = '' } = next.contextData;
        // 这里会将内部的一些子上下文的访问(this.xxx)转换为 __$$context.xxx 的形式
        // 与 Rax 所不同的是，这里不会将最顶层的 this 转换掉
        const customHandlers = {
            expression(input, scope, config) {
                var _a;
                return (0, transformJsExpression_1.transformJsExpr)((0, jsExpression_1.generateExpression)(input, scope), scope, {
                    dontWrapEval: !((_a = config === null || config === void 0 ? void 0 : config.tolerateEvalErrors) !== null && _a !== void 0 ? _a : tolerateEvalErrors),
                    dontTransformThis2ContextAtRootScope: true,
                });
            },
            function(input, scope) {
                return (0, transformThis2Context_1.transformThis2Context)((0, compositeType_1.generateCompositeType)({
                    type: 'JSFunction',
                    value: input.value || 'function () {}',
                }, Scope_1.Scope.createRootScope()), scope, { ignoreRootScope: true });
            },
        };
        const generatorPlugins = {
            handlers: customHandlers,
            tagMapping: (v) => nodeTypeMapping[v] || v,
            tolerateEvalErrors,
        };
        if (next.contextData.useRefApi) {
            generatorPlugins.attrPlugins = [
                (attrData, scope, pluginCfg, nextFunc) => {
                    if (attrData.attrName === 'ref') {
                        return [
                            {
                                name: attrData.attrName,
                                value: `this._refsManager.linkRef('${attrData.attrValue}')`,
                                type: types_1.PIECE_TYPE.ATTR,
                            },
                        ];
                    }
                    return nextFunc ? nextFunc(attrData, scope, pluginCfg) : [];
                },
            ];
        }
        const generator = (0, nodeToJSX_1.createReactNodeGenerator)(generatorPlugins);
        const ir = next.ir;
        const scope = Scope_1.Scope.createRootScope();
        const jsxContent = generator(ir, scope);
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: const_1.REACT_CHUNK_NAME.ClassRenderJSX,
            content: `
        const __$$context = this._context || this;
        const { state } = __$$context;
        return ${jsxContent};
      `,
            linkAfter: [const_1.REACT_CHUNK_NAME.ClassRenderStart, const_1.REACT_CHUNK_NAME.ClassRenderPre],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.COMMON_CHUNK_NAME.CustomContent,
            content: [
                tolerateEvalErrors &&
                    `
          function __$$eval(expr) {
            try {
              return expr();
            } catch (error) {
              ${evalErrorsHandler}
            }
          }

          function __$$evalArray(expr) {
            const res = __$$eval(expr);
            return Array.isArray(res) ? res : [];
          }
      `,
                `
        function __$$createChildContext(oldContext, ext) {
          const childContext = {
            ...oldContext,
            ...ext,
          };
          childContext.__proto__ = oldContext;
          return childContext;
        }
      `,
            ]
                .filter(Boolean)
                .join('\n'),
            linkAfter: [generator_1.COMMON_CHUNK_NAME.FileExport],
        });
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=jsx.js.map