"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDiskPublisher = void 0;
const tslib_1 = require("tslib");
const defaultFs = tslib_1.__importStar(require("fs"));
const types_1 = require("../../types");
const errors_1 = require("../../utils/errors");
const utils_1 = require("./utils");
const createDiskPublisher = (params = {}) => {
    let { project, outputPath = './' } = params;
    const { fs = defaultFs } = params;
    const getProject = () => {
        if (!project) {
            throw new types_1.PublisherError('MissingProject');
        }
        return project;
    };
    const setProject = (projectToSet) => {
        project = projectToSet;
    };
    const getOutputPath = () => {
        return outputPath;
    };
    const setOutputPath = (path) => {
        outputPath = path;
    };
    const publish = (options = {}) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const projectToPublish = options.project || project;
        if (!projectToPublish) {
            throw new types_1.PublisherError('MissingProject');
        }
        const projectOutputPath = options.outputPath || outputPath;
        const overrideProjectSlug = options.projectSlug || params.projectSlug;
        const createProjectFolder = (_a = options.createProjectFolder) !== null && _a !== void 0 ? _a : params.createProjectFolder;
        if (overrideProjectSlug) {
            projectToPublish.name = overrideProjectSlug;
        }
        try {
            yield (0, utils_1.writeFolder)(projectToPublish, projectOutputPath, createProjectFolder, fs);
            return { success: true, payload: projectOutputPath };
        }
        catch (error) {
            throw new types_1.PublisherError((0, errors_1.getErrorMessage)(error) || 'UnknownError');
        }
    });
    return {
        publish,
        getProject,
        setProject,
        getOutputPath,
        setOutputPath,
    };
};
exports.createDiskPublisher = createDiskPublisher;
//# sourceMappingURL=index.js.map