"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFunction = exports.generateExpression = exports.isBroadJSFunction = void 0;
const tslib_1 = require("tslib");
const parser = tslib_1.__importStar(require("@babel/parser"));
const generator_1 = tslib_1.__importDefault(require("@babel/generator"));
const traverse_1 = tslib_1.__importDefault(require("@babel/traverse"));
const t = tslib_1.__importStar(require("@babel/types"));
const lowcode_types_1 = require("@alilc/lowcode-types");
const types_1 = require("../types");
const expressionParser_1 = require("./expressionParser");
const common_1 = require("./common");
function parseFunction(content) {
    try {
        const ast = parser.parse(`(${content});`, {
            plugins: [
                'jsx',
            ],
        });
        let resultNode = null;
        (0, traverse_1.default)(ast, {
            FunctionExpression(path) {
                resultNode = path.node;
                path.stop();
            },
        });
        return resultNode;
    }
    catch (e) {
        throw new expressionParser_1.ParseError(content, e);
    }
}
function transformFuncExpr2MethodMember(methodName, content) {
    const funcNode = parseFunction(content);
    if (funcNode) {
        const targetNode = t.classMethod('method', (methodName && t.identifier(methodName)) || funcNode.id || t.identifier('notDefineName'), funcNode.params, funcNode.body, undefined, undefined, undefined, funcNode.async || undefined);
        const { code: resultCode } = (0, generator_1.default)(targetNode, { sourceMaps: false });
        return resultCode;
    }
    throw new Error('Can not find Function Statement');
}
function getArrowFunction(content) {
    const funcNode = parseFunction(content);
    if (funcNode) {
        const targetNode = t.arrowFunctionExpression(funcNode.params, funcNode.body, funcNode.async || undefined);
        const { code: resultCode } = (0, generator_1.default)(targetNode, { sourceMaps: false });
        return resultCode;
    }
    throw new Error('Can not find Function Statement');
}
function getBodyStatements(content) {
    const funcNode = parseFunction(content);
    if (funcNode) {
        const statements = funcNode.body.body;
        const targetNode = t.program(statements, undefined, 'module', undefined);
        const { code: resultCode } = (0, generator_1.default)(targetNode, { sourceMaps: false });
        return resultCode;
    }
    throw new Error('Can not find Function Statement');
}
/**
 * 是否是广义上的 JSFunction
 * @param value
 * @returns
 */
function isBroadJSFunction(value) {
    return (0, common_1.isJSExpressionFn)(value) || (0, lowcode_types_1.isJSFunction)(value);
}
exports.isBroadJSFunction = isBroadJSFunction;
function generateExpression(value, scope) {
    if ((0, lowcode_types_1.isJSExpression)(value)) {
        const exprVal = value.value.trim();
        if (!exprVal) {
            return 'null';
        }
        const afterProcessWithLocals = (0, expressionParser_1.transformExpressionLocalRef)(exprVal, scope);
        return afterProcessWithLocals;
    }
    throw new types_1.CodeGeneratorError('Not a JSExpression');
}
exports.generateExpression = generateExpression;
function getFunctionSource(cfg) {
    return cfg.source || cfg.value || cfg.compiled;
}
function generateFunction(value, config = {
    name: undefined,
    isMember: false,
    isBlock: false,
    isArrow: false,
    isBindExpr: false,
}) {
    if (isBroadJSFunction(value)) {
        const functionCfg = value;
        const functionSource = getFunctionSource(functionCfg);
        if (config.isMember) {
            return transformFuncExpr2MethodMember(config.name || '', functionSource);
        }
        if (config.isBlock) {
            return getBodyStatements(functionSource);
        }
        if (config.isArrow) {
            return getArrowFunction(functionSource);
        }
        if (config.isBindExpr) {
            return `(${functionSource}).bind(this)`;
        }
        return functionSource;
    }
    if ((0, lowcode_types_1.isJSExpression)(value)) {
        return value.value;
    }
    throw new types_1.CodeGeneratorError('Not a JSFunction or JSExpression');
}
exports.generateFunction = generateFunction;
//# sourceMappingURL=jsExpression.js.map